/**
 * @file tuya_cnt_rst.c
 * @brief
 * @date 2019-04-25
 *
 * @copyright Copyright (c) {2018-2020} 涂鸦科技 www.tuya.com
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tuya_os_adapter.h"
#include "tuya_cnt_rst.h"
#include "mem_pool.h"
#include "uni_log.h"
#include "sys_timer.h"
#include "tuya_uf_db.h"
#include "cJSON.h"
#include "tuya_hal_system.h"
#include "tuya_device.h"

#define TUYA_CNT_RST_TIMEOUT         5000            /* 上电计数清除时间 单位 MS */
#define TUYA_RST_CNT_KEY "tuya_rst_cnt_key"
typedef struct {
    BOOL_T enable;
    UINT8_T rst_num;
    TUYA_CNT_RST_INFORM_CB inform_cb;
}TUYA_CNT_RST_S;

TUYA_CNT_RST_S tuya_cnt_rst = { 0 };


STATIC OPERATE_RET tuya_rst_cnt_set(UINT8_T cnt)
{
    CHAR_T out[32] = { 0 };
    snprintf(out, 32, "{\"cnt\":%d}", cnt);
    PR_DEBUG("rst_cnt_set:%s", out);
    uFILE* fp = NULL;
    fp = ufopen(TUYA_RST_CNT_KEY,"w+");
    if(NULL == fp) {
        PR_ERR("ufopen err");
        return OPRT_COM_ERROR;
    }

    INT_T Num = ufwrite(fp, (UCHAR_T *)out, strlen(out));
    ufclose(fp);
    if( Num <= 0) {
        PR_ERR("ufwrite fail:%d",Num);
        return OPRT_COM_ERROR;
    }
    //PR_DEBUG("uf write ok %d",Num);
    return OPRT_OK;
}

STATIC UINT8_T tuya_rst_cnt_get()
{
    uFILE * fp = ufopen(TUYA_RST_CNT_KEY,"r");
    if(NULL == fp) {     /* 如果无法打开 */
        PR_ERR("ufopen fail");
        goto GET_ERR_EXIT;
    }
    INT_T buf_len = 32;
    CHAR_T * buf = (CHAR_T *)Malloc(buf_len);
    if(NULL == buf) {
        PR_ERR("malloc fail");
        ufclose(fp);
        goto GET_ERR_EXIT;
    }

    memset(buf,0,buf_len);

    INT_T Num = ufread(fp, (UCHAR_T *)buf,buf_len);
    ufclose(fp);
    if(Num <= 0) {
        Free(buf);
        PR_ERR("ufread err:%d",Num);
        goto GET_ERR_EXIT;
    }
    PR_DEBUG("get_buf:%s", buf);
    cJSON *root = ty_cJSON_Parse(buf);
    Free(buf);
    if (NULL == root) {
        goto GET_ERR_EXIT;
    }
    cJSON *js_rst = ty_cJSON_GetObjectItem(root, "cnt");
    UINT8_T cnt = 0;
    if (NULL != js_rst && js_rst->type == ty_cJSON_Number) {
        cnt = js_rst->valueint;
    }
    ty_cJSON_Delete(root);
    return cnt;
GET_ERR_EXIT:
    PR_NOTICE("rst_cnt_get err!");
    return 0;
}

STATIC VOID tuya_cnt_rst_timeout_cb(UINT_T timerID, PVOID_T pTimerArg)
{
    tuya_rst_cnt_set(0);
    //PR_DEBUG("tuya_cnt_rst -> 0");
}

VOID tuya_cnt_rst_judge(VOID)
{
    if (!tuya_cnt_rst.enable) {
        return;
    }
    UINT8_T cur_rst_cnt = tuya_rst_cnt_get();         /* 灯重置次数的计数值 */
#if ENABLE_TUYA_OS_3
    if( TUYA_RESET_REASON_POWERON == tuya_hal_system_get_rst_info() ) {
#else
    if( TY_RST_POWER_OFF == tuya_hal_system_get_rst_info() ) {
#endif
        cur_rst_cnt += 1;
    }
    if (cur_rst_cnt >= tuya_cnt_rst.rst_num) {
        tuya_rst_cnt_set(0);
        if (tuya_cnt_rst.inform_cb) {
            tuya_cnt_rst.inform_cb();
        }
    } else {
        tuya_rst_cnt_set(cur_rst_cnt);
        TIMER_ID timeout_timer;
        OPERATE_RET op_ret;
        op_ret = sys_add_timer(tuya_cnt_rst_timeout_cb, NULL, &timeout_timer);
        if(OPRT_OK != op_ret) {
            PR_ERR("add timer err:%d", op_ret);
        } else {
            sys_start_timer(timeout_timer, TUYA_CNT_RST_TIMEOUT, TIMER_ONCE);        /* 启动一个timer，定时满足后清除cnt值 */
        }
        //PR_NOTICE("current rst count: %d", cur_rst_cnt);
    }
}

OPERATE_RET tuya_set_cnt_rst_mode(IN CONST UINT8_T rst_num, IN CONST TUYA_CNT_RST_INFORM_CB cb)
{
    if (rst_num <= 1) {
        PR_ERR("rst_num invalid");
        return OPRT_INVALID_PARM;
    }
    tuya_cnt_rst.enable = TRUE;
    tuya_cnt_rst.rst_num = rst_num;
    tuya_cnt_rst.inform_cb = cb;
    PR_DEBUG("set_cnt_rst_mode %d ok.", rst_num);
    return OPRT_OK;
}
